/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.spagosoa.smx.components.talend;

import it.eng.spagosoa.metadb.dbutils.HibernateUtil;
import it.eng.spagosoa.metadb.model.XmlNamespace;

import java.awt.SystemColor;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.messaging.RobustInOnly;
import javax.xml.transform.Source;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.servicemix.JbiConstants;
import org.apache.servicemix.MessageExchangeListener;
import org.apache.servicemix.components.util.ComponentSupport;
import org.apache.servicemix.components.util.TransformComponentSupport;
import org.apache.servicemix.jbi.FaultException;
import org.apache.servicemix.jbi.jaxp.SourceTransformer;
import org.apache.servicemix.jbi.jaxp.StringSource;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Node;
import org.dom4j.XPath;
import org.dom4j.xpath.DefaultXPath;
import org.hibernate.Session;

public class TalendCaller extends TransformComponentSupport{

	private static final Log log = LogFactory.getLog(TalendCaller.class);
	private Class talendJobClass = null; 
	private String  talendJobClassName = null;
	private String  talendJobClassDescription = null;
	private HashMap parameters = null;
	private Method mainMethod = null;
	
	public static Map<String, String> namespaceURI = null;
	
	public void start() throws JBIException {
    	
		try{
			log.debug("Talend Caller::start() -> Starting Taleb Caller with Job Class ["+talendJobClassName+"]");
			this.talendJobClass = Class.forName(talendJobClassName);
			this.mainMethod = this.talendJobClass.getMethod("main", new Class[]{String[].class});
			log.debug("Talend Caller::start() -> Talend Job ["+talendJobClassName+"] started ");
			if (namespaceURI == null)
				initNameSpaceURI();
			
		}catch (Exception e) {
			log.error("Talend Caller::start() ->   Transformer Class Istantiation Failed");
			log.error(e);
			throw new JBIException(e.getMessage());
		}
    }
	
	public synchronized static void initNameSpaceURI(){
		Session aSession = null;
		try {
			namespaceURI = new HashMap<String, String>();
			aSession = HibernateUtil.getSessionFactory().openSession();
			List<XmlNamespace> lNs = aSession.createQuery(" from it.eng.spagosoa.metadb.model.XmlNamespace").list();
			for (XmlNamespace ns : lNs) {
				namespaceURI.put(ns.getPrefix(), ns.getValue());
			}
		}catch(Throwable t){
			t.printStackTrace();
		}finally{
			aSession.close();
		}
	}
	
	public static Map<String, String> getNamespaceURI(){
		return namespaceURI;
	}
	
	@Override
	protected boolean transform(MessageExchange exchange, NormalizedMessage in,
			NormalizedMessage out) throws Exception {
		try{	
			SourceTransformer sourceTransformer = new SourceTransformer();
	        String flussoXMLInput = sourceTransformer.contentToString(in); 
	        
			Object[] mainMethodParams  = new Object[1];
			String[] args = new String[parameters.size()];
			Set<String> keySet = parameters.keySet();
			int index = 0;
			Document document = null;
			XPath xPathExpression = null;
			for(String key:keySet) {
				if(!((String)parameters.get(key)).startsWith("/"))
					args[index++] = "--context_param " + key + "=" + parameters.get(key);
				else {
					document = DocumentHelper.parseText(flussoXMLInput);
					xPathExpression = new DefaultXPath((String)parameters.get(key));
					//xPathExpression.setNamespaceURIs(getNamespaceURI());
					String extractedData = xPathExpression.valueOf(document);
					if (extractedData != null && !extractedData.equals(""))
						args[index++] = "--context_param " + key + "=" + extractedData;
				}
			}
			mainMethodParams[0] = args;
 			mainMethod.invoke(null, mainMethodParams);
			Source outMsg = null;
			outMsg = toXMLSource("<OK>Talend Caller is OK </OK>");
			if (outMsg != null) {
	              
	                out.setContent(outMsg);
			}else{
				Fault fault = exchange.createFault();
	        	fault.setContent(new StringSource("<ERROR>Out Msg is null </ERROR>"));
	        	fault.setProperty("org.apache.servicemix.xml", in.getContent());
	        	throw new FaultException("Talend Caller Failed ", exchange, fault);
			}
			return true;
		}catch (Throwable t) {
    		log.error("Exception in Talend Caller");
    		log.error(t);
    		Fault fault = exchange.createFault();
        	fault.setContent(new StringSource("<ERROR>An error is occured in Talend Caller</ERROR>"));
        	fault.setProperty("org.apache.servicemix.xml", in.getContent());
        	throw new FaultException("Talend Caller Failed ", exchange, fault);
		}
	}
	
	protected Source toXMLSource(String s) throws Exception {
		return new StringSource(s);
    }

	public String getTalendJobClassName() {
		return talendJobClassName;
	}

	public void setTalendJobClassName(String talendJobClassName) {
		this.talendJobClassName = talendJobClassName;
	}

	public String getTalendJobClassDescription() {
		return talendJobClassDescription;
	}

	public void setTalendJobClassDescription(String talendJobClassDescription) {
		this.talendJobClassDescription = talendJobClassDescription;
	}

	public HashMap getParameters() {
		return parameters;
	}

	public void setParameters(HashMap parameters) {
		this.parameters = parameters;
	}
	
    
	
}